@@ -15,7 +15,9 @@
 # KIND, either express or implied.  See the License for the
 # specific language governing permissions and limitations
 # under the License.
-from typing import TYPE_CHECKING, Optional, Sequence
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Sequence
 
 from airflow.models import BaseOperator
 from airflow.providers.cncf.kubernetes.hooks.kubernetes import KubernetesHook
@@ -33,17 +35,12 @@ class SparkKubernetesOperator(BaseOperator):
         https://github.com/GoogleCloudPlatform/spark-on-k8s-operator/blob/v1beta2-1.1.0-2.4.5/docs/api-docs.md#sparkapplication
 
     :param application_file: Defines Kubernetes 'custom_resource_definition' of 'sparkApplication' as either a
-        path to a '.json' file or a JSON string.
-    :type application_file:  str
+        path to a '.yaml' file, '.json' file, YAML string or JSON string.
     :param namespace: kubernetes namespace to put sparkApplication
-    :type namespace: str
     :param kubernetes_conn_id: The :ref:`kubernetes connection id <howto/connection:kubernetes>`
         for the to Kubernetes cluster.
-    :type kubernetes_conn_id: str
     :param api_group: kubernetes api group of sparkApplication
-    :type api_group: str
     :param api_version: kubernetes api version of sparkApplication
-    :type api_version: str
     """
 
     template_fields: Sequence[str] = ('application_file', 'namespace')
@@ -54,7 +51,7 @@ class SparkKubernetesOperator(BaseOperator):
         self,
         *,
         application_file: str,
-        namespace: Optional[str] = None,
+        namespace: str | None = None,
         kubernetes_conn_id: str = 'kubernetes_default',
         api_group: str = 'sparkoperator.k8s.io',
         api_version: str = 'v1beta2',
@@ -66,14 +63,15 @@ class SparkKubernetesOperator(BaseOperator):
         self.kubernetes_conn_id = kubernetes_conn_id
         self.api_group = api_group
         self.api_version = api_version
+        self.plural = "sparkapplications"
 
-    def execute(self, context: 'Context'):
-        self.log.info("Creating sparkApplication")
+    def execute(self, context: Context):
         hook = KubernetesHook(conn_id=self.kubernetes_conn_id)
+        self.log.info("Creating sparkApplication")
         response = hook.create_custom_object(
             group=self.api_group,
             version=self.api_version,
-            plural="sparkapplications",
+            plural=self.plural,
             body=self.application_file,
             namespace=self.namespace,
         )
